@@ -16,7 +16,11 @@
  */
 package org.apache.wicket.request.cycle;
 
+import java.util.Optional;
+
 import org.apache.wicket.Application;
+import org.apache.wicket.IMetadataContext;
+import org.apache.wicket.IWicketInternalException;
 import org.apache.wicket.MetaDataEntry;
 import org.apache.wicket.MetaDataKey;
 import org.apache.wicket.Page;
@@ -35,7 +39,8 @@ import org.apache.wicket.request.IRequestCycle;
 import org.apache.wicket.request.IRequestHandler;
 import org.apache.wicket.request.IRequestMapper;
 import org.apache.wicket.request.Request;
-import org.apache.wicket.request.RequestHandlerStack;
+import org.apache.wicket.request.RequestHandlerExecutor;
+import org.apache.wicket.request.RequestHandlerExecutor.ReplaceHandlerException;
 import org.apache.wicket.request.Response;
 import org.apache.wicket.request.Url;
 import org.apache.wicket.request.UrlRenderer;
@@ -47,6 +52,7 @@ import org.apache.wicket.request.resource.IResource;
 import org.apache.wicket.request.resource.ResourceReference;
 import org.apache.wicket.request.resource.caching.IStaticCacheableResource;
 import org.apache.wicket.util.lang.Args;
+import org.apache.wicket.util.lang.Exceptions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -66,7 +72,7 @@ import org.slf4j.LoggerFactory;
  * @author Matej Knopp
  * @author igor.vaynberg
  */
-public class RequestCycle implements IRequestCycle, IEventSink
+public class RequestCycle implements IRequestCycle, IEventSink, IMetadataContext<Object, RequestCycle>
 {
 	private static final Logger log = LoggerFactory.getLogger(RequestCycle.class);
 
@@ -115,7 +121,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	/** the time that this request cycle object was created. */
 	private final long startTime;
 
-	private final RequestHandlerStack requestHandlerExecutor;
+	private final RequestHandlerExecutor requestHandlerExecutor;
 
 	private Response activeResponse;
 
@@ -203,82 +209,6 @@ public class RequestCycle implements IRequestCycle, IEventSink
 		return retries;
 	}
 
-	/**
-	 * Processes the request.
-	 * 
-	 * @return <code>true</code> if the request resolved to a Wicket request, <code>false</code>
-	 *         otherwise.
-	 */
-	public boolean processRequest()
-	{
-		try
-		{
-			set(this);
-			listeners.onBeginRequest(this);
-			onBeginRequest();
-			IRequestHandler handler = resolveRequestHandler();
-			if (handler != null)
-			{
-				execute(handler);
-				return true;
-			}
-
-			// Did not find any suitable handler, thus not executing the request
-			log.debug(
-				"No suitable handler found for URL {}, falling back to container to process this request",
-				request.getUrl());
-		}
-		catch (Exception e)
-		{
-			IRequestHandler handler = handleException(e);
-			if (handler != null)
-			{
-				listeners.onExceptionRequestHandlerResolved(this, handler, e);
-				executeExceptionRequestHandler(handler, getExceptionRetryCount());
-				listeners.onRequestHandlerExecuted(this, handler);
-			}
-			else
-			{
-				log.error("Error during request processing. URL=" + request.getUrl(), e);
-			}
-			return true;
-		}
-		finally
-		{
-			set(null);
-		}
-		return false;
-	}
-
-	/**
-	 * Executes a request handler and fires pre/post listener methods
-	 * 
-	 * @param handler
-	 */
-	private void execute(IRequestHandler handler)
-	{
-		Args.notNull(handler, "handler");
-
-		try
-		{
-			listeners.onRequestHandlerResolved(this, handler);
-			requestHandlerExecutor.execute(handler);
-			listeners.onRequestHandlerExecuted(this, handler);
-		}
-		catch (RuntimeException e)
-		{
-			IRequestHandler replacement = requestHandlerExecutor.resolveHandler(e);
-			if (replacement != null)
-			{
-				execute(replacement);
-			}
-			else
-			{
-				throw e;
-			}
-		}
-	}
-
 	/**
 	 * Convenience method that processes the request and detaches the {@link RequestCycle}.
 	 * 
@@ -299,40 +229,139 @@ public class RequestCycle implements IRequestCycle, IEventSink
 		return result;
 	}
 
+	/**
+	 * Processes the request.
+	 * 
+	 * @return <code>true</code> if the request resolved to a Wicket request, <code>false</code>
+	 *         otherwise.
+	 */
+	public boolean processRequest()
+	{
+		try
+		{
+			set(this);
+			listeners.onBeginRequest(this);
+			onBeginRequest();
+			IRequestHandler handler = resolveRequestHandler();
+			if (handler == null)
+			{
+				// Did not find any suitable handler, thus not executing the request
+				log.debug(
+					"No suitable handler found for URL {}, falling back to container to process this request",
+					request.getUrl());
+			}
+			else
+			{
+				execute(handler);
+				return true;
+			}
+		}
+		catch (Exception exception)
+		{
+			return executeExceptionRequestHandler(exception, getExceptionRetryCount());
+		}
+		finally
+		{
+			try
+			{
+				listeners.onEndRequest(this);
+				onEndRequest();
+			}
+			catch (RuntimeException e)
+			{
+				log.error("Exception occurred during onEndRequest", e);
+			}
+
+			set(null);
+		}
+
+		return false;
+	}
 
 	/**
+	 * Execute a request handler and notify registered {@link IRequestCycleListener}s.
 	 * 
 	 * @param handler
+	 */
+	private void execute(IRequestHandler handler)
+	{
+		Args.notNull(handler, "handler");
+
+		while (handler != null) {
+			try
+			{
+				listeners.onRequestHandlerResolved(this, handler);
+				IRequestHandler next = requestHandlerExecutor.execute(handler);
+				listeners.onRequestHandlerExecuted(this, handler);
+				
+				handler = next;
+			}
+			catch (RuntimeException e)
+			{
+				ReplaceHandlerException replacer = Exceptions.findCause(e, ReplaceHandlerException.class);
+
+				if (replacer == null)
+				{
+					throw e;
+				}
+
+				if (replacer.getRemoveScheduled())
+				{
+					requestHandlerExecutor.schedule(null);
+				}
+
+				handler = replacer.getReplacementRequestHandler();
+			}
+		}
+	}
+
+	/**
+	 * Process the given exception.
+	 * 
+	 * @param exception
 	 * @param retryCount
 	 */
-	private void executeExceptionRequestHandler(final IRequestHandler handler, final int retryCount)
+	private boolean executeExceptionRequestHandler(Exception exception, int retryCount)
 	{
+		IRequestHandler handler = handleException(exception);
+		if (handler == null)
+		{
+			log.error("Error during request processing. URL=" + request.getUrl(), exception);
+			return false;
+		}
+
 		scheduleRequestHandlerAfterCurrent(null);
 
 		try
 		{
-			requestHandlerExecutor.execute(handler);
+			listeners.onExceptionRequestHandlerResolved(this, handler, exception);
+
+			execute(handler);
+			
+			return true;
 		}
 		catch (Exception e)
 		{
 			if (retryCount > 0)
 			{
-				IRequestHandler next = handleException(e);
-				if (next != null)
-				{
-					executeExceptionRequestHandler(next, retryCount - 1);
-					return;
-				}
+				return executeExceptionRequestHandler(exception, retryCount - 1);
+			}
+			else
+			{
+				log.error("Exception retry count exceeded", e);
+				return false;
 			}
-			log.error("Error during processing error message", e);
 		}
 	}
 
 	/**
 	 * Return {@link IRequestHandler} for the given exception.
 	 * 
-	 * @param e
+	 * @param e exception to handle
 	 * @return RequestHandler instance
+	 *
+	 * @see IRequestCycleListener#onException(RequestCycle, Exception)
+	 * @see IExceptionMapper#map(Exception)
 	 */
 	protected IRequestHandler handleException(final Exception e)
 	{
@@ -394,6 +423,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 * @throws IllegalArgumentException
 	 * @see MetaDataKey
 	 */
+	@Override
 	public final <T> RequestCycle setMetaData(final MetaDataKey<T> key, final T object)
 	{
 		metaData = key.set(metaData, object);
@@ -411,6 +441,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 * @return The metadata or null if no metadata was found for the given key
 	 * @see MetaDataKey
 	 */
+	@Override
 	public final <T> T getMetaData(final MetaDataKey<T> key)
 	{
 		return key.get(metaData);
@@ -626,16 +657,6 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 */
 	public void onDetach()
 	{
-		try
-		{
-			onEndRequest();
-			listeners.onEndRequest(this);
-		}
-		catch (RuntimeException e)
-		{
-			log.error("Exception occurred during onEndRequest", e);
-		}
-
 		try
 		{
 			requestHandlerExecutor.detach();
@@ -664,19 +685,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 */
 	private void handleDetachException(RuntimeException exception) 
 	{
-		boolean isBufferedResponse = true;
-		if (Application.exists())
-		{
-			isBufferedResponse = Application.get().getRequestCycleSettings().getBufferResponse();
-		}
-
-		//if application is using a buffered response strategy,
-		//then we display exception to user.
-		if (isBufferedResponse) 
-		{
-			throw exception;
-		}
-		else 
+		if (!(exception instanceof IWicketInternalException))
 		{
 			log.error("Error detaching RequestCycle", exception);
 		}
@@ -780,6 +789,10 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 */
 	protected void onEndRequest()
 	{
+		if (Session.exists())
+		{
+			Session.get().endRequest();
+		}
 	}
 
 	/**
@@ -825,7 +838,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	}
 
 	/**
-	 * @see RequestHandlerStack#getActive()
+	 * @see RequestHandlerExecutor#getActive()
 	 * @return active handler on executor
 	 */
 	public IRequestHandler getActiveRequestHandler()
@@ -834,7 +847,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	}
 
 	/**
-	 * @see RequestHandlerStack#next()
+	 * @see RequestHandlerExecutor#next()
 	 * @return the handler scheduled to be executed after current by the executor
 	 */
 	public IRequestHandler getRequestHandlerScheduledAfterCurrent()
@@ -843,7 +856,7 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	}
 
 	/**
-	 * @see RequestHandlerStack#replaceAll(IRequestHandler)
+	 * @see RequestHandlerExecutor#replaceAll(IRequestHandler)
 	 * @param handler
 	 */
 	public void replaceAllRequestHandlers(final IRequestHandler handler)
@@ -855,35 +868,37 @@ public class RequestCycle implements IRequestCycle, IEventSink
 	 * Finds a IRequestHandler which is either the currently executing handler or is scheduled to be
 	 * executed.
 	 * 
-	 * @return the found IRequestHandler or {@code null}
+	 * @return the found IRequestHandler or {@link Optional#empty()}
 	 */
-	public <T extends IRequestHandler> T find(final Class<T> type)
+	@SuppressWarnings("unchecked")
+	public <T extends IRequestHandler> Optional<T> find(final Class<T> type)
 	{
 		if (type == null)
 		{
-			return null;
+			return Optional.empty();
 		}
 
 		IRequestHandler result = getActiveRequestHandler();
-
-		if (result == null || type.isAssignableFrom(result.getClass()) == false)
+		if (type.isInstance(result))
 		{
-			result = getRequestHandlerScheduledAfterCurrent();
-			if (result == null || type.isAssignableFrom(result.getClass()) == false)
-			{
-				result = null;
-			}
+			return (Optional<T>)Optional.of(result);
+		}
+		
+		result = getRequestHandlerScheduledAfterCurrent();
+		if (type.isInstance(result))
+		{
+			return (Optional<T>)Optional.of(result);
 		}
 
-		return (T)result;
+		return Optional.empty();
 	}
 
 	/**
-	 * Adapts {@link RequestHandlerStack} to this {@link RequestCycle}
+	 * Adapts {@link RequestHandlerExecutor} to this {@link RequestCycle}
 	 * 
 	 * @author Igor Vaynberg
 	 */
-	private class HandlerExecutor extends RequestHandlerStack
+	private class HandlerExecutor extends RequestHandlerExecutor
 	{
 
 		@Override
